#!/usr/bin/perl

#use feature ':5.10';
use strict 'vars';
use warnings;
use Getopt::Long;
use Pod::Usage;


my $i_dir;
my $i_aln_fasta_file;

my $options = GetOptions ( "input=s"	=> \$i_aln_fasta_file);

($i_aln_fasta_file) or pod2usage("error: -input-dir is mandatory!\n");
(-e $i_aln_fasta_file) or pod2usage("error: no such directory: $i_dir\n");


my ($fasta_href, $order_aref, $header_href) = read_fasta_file ($i_aln_fasta_file);

foreach my $id (keys %{$fasta_href})
{
	$fasta_href->{$id} =~ s/[\.]//g; ## remove dots
}

my $fasta_name = $i_aln_fasta_file;
$fasta_name =~ s/\.aln\.fa$/\.fa/g;
$fasta_name =~ s/\.aln\.fasta$/\.fasta/g;
write_hash_to_fasta($fasta_name, $fasta_href);
##########################################################################################
# This method takes an array, which contains a string for each element that represents
# one line in the output file and writes the information in the array to  a file with the 
# given name.
# Input: 
#	filename 		The name of the file
#	output_array	Contents to write to the file, line by line
#	append			Append to existing file
##########################################################################################
sub write_file_from_array{
	my($filename, $output_array, $append) = @_;
	
	if($append){
		open(OUT_HANDLE, ">>$filename") || die "couldn't open the file for writing in package Tools,".
								"sub write_file_from_array: $filename\n";
	} else{
	
		open(OUT_HANDLE, ">$filename") || die "couldn't open the file for writing in package Tools,".
								"sub write_file_from_array: $filename\n";
	}
		
	chomp(@{$output_array});						
	foreach my $line (@{$output_array}){
		
		print OUT_HANDLE $line."\n";
	}
}
	

##################################################################################
# This method writes the data in a hash that has sequences as a value to a fasta file.
# NOTE: This method can be used in combination with the read_fasta_file* methods to
# convert a multi-lined fasta file to a single lined fasta file. First read the
# fasta file to a hash and then call this method to write the hash to either the
# same file or a different one.
#
# Input:
#		filename	The name of the fasta file with path
#		data_href	The reference to the hash containing the fasta data
# Output: none
##################################################################################
sub write_hash_to_fasta{
	my($filename, $data_href) = @_;

	open(OUT_HANDLE, ">$filename") || die "couldn't open the file for writing in package Tools,".
								"sub write_file_from_array: $filename\n";

	foreach my $key (sort(keys (%{$data_href}))){

		print OUT_HANDLE ">$key\n";
		print OUT_HANDLE $data_href->{$key}."\n";
	}
	close(OUT_HANDLE);
	return 1;
}

##################################################################################
# This method parses a standard fasta file and returns the sequences in a hash
# with the name as the key. The name of the sequence is taken from the header,
# which is the first word after the '>' symbol until the first space.
# The first word may contain any symbol (except spaces of course).
# Furthermore, the method deals with multiple lines, and returns a single sequence.
# Input:
#		file		The name of the fasta file
# Output:
#		An array with
#	(1)	A hash reference where the key is the item id and the value is the
#		sequence as a string.
#	(2)	An array reference including the ids in the order they are given in the
#		input file, $file. This information is necessary if you need the exact
#		order, which is not given in the hash.
#	(3) A hash reference of the rest of the header line in the fasta file (after the IDs)
##################################################################################
sub read_fasta_file{
	my($file) = @_;
	my $FUNCTION = "read_fasta_file in Sequences.pm";

	my $id 			= "";
	my $seqstring 	= "";
	my %fasta 		= ();
	my %header		= ();
	my @order 		= ();
	my $line 		= "";
	open(IN_HANDLE, "<$file") || die "ERROR in $FUNCTION: ".
									 "Couldn't open the following file in package Tool,".
									 " sub read_fasta_file: $file\n";

	while($line = <IN_HANDLE>){
		chomp($line);

		# header (can contain one space after > symbol)
		if($line =~ /^\>\s?(\S+)\s*([\S*\s*]*)/){
			if($id){
				if (defined $fasta{$id} and ($fasta{$id} ne $seqstring)) {
					die "ERROR in $FUNCTION: ".
						"multiple sequence id '$id', consider using function " .
						"read_fasta_with_nonunique_headers instead of read_fasta_file";
				}
				$seqstring =~ s/\s*//g; ## do not allow spaces in sequence
				$fasta{$id} = $seqstring;
				$seqstring = "";
			}
			$id = $1;
			$header{$id} = $2;
			push(@order, $id);
		} else {
				$seqstring .= $line if ($id);
		}
	}

	if($id){
				if (defined $fasta{$id} and ($fasta{$id} ne $seqstring)) {
					die "ERROR in $FUNCTION: ".
						"multiple sequence id '$id', consider using function " .
						"read_fasta_with_nonunique_headers instead of read_fasta_file";
				}
				$seqstring =~ s/\s*//g; ## do not allow spaces in sequence
				$fasta{$id} = $seqstring;
				$seqstring = "";
	}
	return (\%fasta, \@order, \%header);
}
