#!/usr/bin/env python

__author__ = 'Alexander Junge (alexander.junge@gmail.com)'

import argparse
import os
import collections
import sys


def is_valid_path(arg):
    if not os.path.exists(arg):
        parser.error("Given path %s does not exist" % arg)
    return arg


parser = argparse.ArgumentParser(description=
                                 """Computes the F2-measure for a clustering produced by RNAalignClust""")
parser.add_argument('clusterfile', type=is_valid_path,
                    help='the clustering to be assessed. IDs of objects in one cluster are space separated and listed '
                         'in a single line,')
parser.add_argument('classfile', type=is_valid_path,
                    help='contains one line per object ID in the data set defining the gold standard family of each '
                         'cluster. Used to interfere the IDs of objects belonging to each gold standard cluster.')
args = parser.parse_args()

cluster_file_name = args.clusterfile
class_file_name = args.classfile

gold_cluster_id_to_objects = collections.defaultdict(set)
with open(class_file_name, 'r') as class_file:
    for line_number, line in enumerate(class_file):
        gold_cluster = line.strip()
        gold_cluster_id_to_objects[gold_cluster].add(str(line_number))

clusters = []
with open(cluster_file_name, 'r') as cluster_file:
    for line_number, line in enumerate(cluster_file):
        objects = line.strip().split(' ')
        clusters.append(set(objects))

f_measure = 0.0
gold_size_all = 0.0
for gold_cluster in gold_cluster_id_to_objects.values():
    gold_size = len(gold_cluster)
    gold_size_all += gold_size
    f_measure_gold_cluster = 0.0
    for result_cluster in clusters:
        result_size = len(result_cluster)
        overlap_size = len(gold_cluster.intersection(result_cluster))
        denominator = gold_size + result_size
        curr_f = 2 * overlap_size / float(denominator)
        if curr_f > f_measure_gold_cluster:
            f_measure_gold_cluster = curr_f
    f_measure += gold_size * f_measure_gold_cluster
f_measure /= gold_size_all

sys.stdout.write(str(f_measure) + "\n")