__author__ = 'Alexander Junge <alexander.junge@gmail.com>'

import sys
import argparse

from Bio import SeqIO


def is_supported_sequence_file_format(arg):
    if arg.lower() == 'stockholm':
        return 'stockholm'
    if arg.lower() == 'fasta':
        return 'fasta'
    else:
        parser.error("Given alignment file type is not supported." % arg)


parser = argparse.ArgumentParser()
argparse.ArgumentParser(description=
                        '''Counts the number of sequences present in a file in FASTA or
                        Stockholm format.
                        The file is read from stdin and the number of sequences is written to stdout.''')
parser.add_argument('sequence_format', type=is_supported_sequence_file_format,
                    help='format of the sequences read from stdin - "Stockholm" or "FASTA" format')
args = parser.parse_args()
sequence_format = args.sequence_format


# FASTA file is read from stdin
input_file_handler = sys.stdin

sequence_count = 0
for record in SeqIO.parse(input_file_handler, sequence_format):
    sequence_count += 1

sys.stdout.write(str(sequence_count) + '\n')