# run GC here
if [ "$#" -ne 2 ]; then
    echo "Illegal number of parameters expected 2"
    exit 1
fi

gzspan=$1
myfasta=$2
echo "gzspan: $gzspan"

if [ ! -f $gzspan ] || [[  $gzspan != *.gz ]]
  then
    echo "Zipped gspan not available or have wrong suffix... exit: $gzspan"
    exit 1
fi

if [ ! -f $myfasta ] || [[  $myfasta != *.fa ]]
  then
    echo "Fasta file not available or have wrong suffix... exit: $myfasta"
    exit 1
fi

# varnames, unzip...
inputdir=$(dirname $gzspan)
echo $inputdir
# $inputdir=${inputdir%/}  # remove tailing /
gspanname=$(basename $gzspan ".gz")
gunzip --keep -f $gzspan

# Comaptibility check
zcat $gzspan | grep "t #" |  awk '{print $4}' | sed 's/_filtered//' | sed 's/.folded//' > "$inputdir/gspan_ids.txt"
cat $myfasta | grep ">" | awk '{print $2}'  | sed 's/_rep//' > "$inputdir/fasta_ids.txt"


diff_ids=$(diff -w "$inputdir/gspan_ids.txt"  "$inputdir/fasta_ids.txt")

if [ "$diff_ids" != "" ]
  then
    echo "ERROR: gspan and fasta ids not match! Diff is:"
    echo $diff_ids
    exit -1
fi



featbin="$inputdir/$gspanname.feature_bin"

# remove old result
[ -f "$featbin" ] && rm $featbin

#run NSPDK it should generat feature and feature_bin files
'/usr/local/user/GraphClust-0.7.6/bin/NSPDK' -R 3 -D 3 -gt DIRECTED -fg "$inputdir/$gspanname" -of

if [ ! -f $featbin ]
  then
    echo "Feature_bin not available exit: $featbin"
    exit 1
fi

echo "Feature bin is:"
echo $featbin

# Create GC root directories
rootdir="$inputdir/$gspanname-ROOT" 
echo $rootdir
mkdir $rootdir
mkdir $rootdir/AC
mkdir $rootdir/GC



config_file='/home/miladim/projects/AlignClust-Evaluation/data/GraphClust/configs/config.all_default_noabst_numcluster500_group_win_size_exterme_global'

# Run GraphClust Pipeline no sge
echo "Run this command for both AC and GC:"
echo "myfasta=$myfasta ;" '/usr/local/user/GraphClust-0.7.6/bin/MASTER_GraphClust.pl' --fasta \$myfasta --root-dir "$rootdir/AC/" --config "$config_file"
echo "myfasta=$myfasta ;" '/usr/local/user/GraphClust-0.7.6/bin/MASTER_GraphClust.pl' --fasta \$myfasta --root-dir "$rootdir/GC/" --config "$config_file"

# read -p "Cleaning up root direcory, press enter to continue" yn
# rm SGE_LOG/stage[5-9]*
# rm -rf SVECTOR/sge_log_stage5.1/
# rm SVECTOR/blacklist.no_model SVECTOR/data.svector.blacklist.1 SVECTOR/data.svector.1.fast_cluster  SVECTOR/data.svector.1.fast_cluster_sim SVECTOR/data.svector.fast_cluster.1.DONE;
# mv SVECTOR/data.svector SVECTOR/data.svector.orig
# cp $featbin SVECTOR/data.svector
#rerun GC here
# for c in *[0-9]; do cat $c/cluster.all | awk -v c=$c '{print c,$11}' | sort; done > result.txt
