import itertools 

def jaccard(labels1, labels2):
    """
    Computes the Jaccard similarity between two sets of clustering labels.

    The value returned is between 0 and 1, inclusively. A value of 1 indicates
    perfect agreement between two clustering algorithms, whereas a value of 0
    indicates no agreement. For details on the Jaccard index, see:
    http://en.wikipedia.org/wiki/Jaccard_index

    Example:
    labels1 = [1, 2, 2, 3]
    labels2 = [3, 4, 4, 4]
    print jaccard(labels1, labels2)

    @param labels1 iterable of cluster labels
    @param labels2 iterable of cluster labels
    @return the Jaccard similarity value
    """
    n11 = n10 = n01 = 0
    n = len(labels1)
    # TODO: Throw exception if len(labels1) != len(labels2)
    for i, j in itertools.combinations(xrange(n), 2):
        comembership1 = labels1[i] == labels1[j]
        comembership2 = labels2[i] == labels2[j]
        if comembership1 and comembership2:
            n11 += 1
        elif comembership1 and not comembership2:
            n10 += 1
        elif not comembership1 and comembership2:
            n01 += 1
    return float(n11) / (n11 + n10 + n01)

def counts(clusters_true, clusters_pred):
    """
    Computes the TP TN FP FN between two sets of clustering ids or labels.

    
    @param clusters_true iterable of cluster labels
    @param clusters_pred iterable of cluster labels
    @return counts in a dict format
    """
    TP = TN = FP = FN = 0
    n = len(clusters_true)
    assert (len(clusters_true) == len(clusters_pred))
    for i, j in itertools.combinations(xrange(n), 2):
        comembership_true = clusters_true[i] == clusters_true[j]
        comembership_pred = clusters_pred[i] == clusters_pred[j]
        if comembership_true and comembership_pred:
            TP += 1
        elif comembership_true and not comembership_pred:
            FN += 1
        elif not comembership_true and comembership_pred:
            FP += 1
        else:
            TN += 1
    return {"TP": TP, "TN": TN, "FP": FP, "FN": FN}
