#!/usr/bin/env python
# Matthew's correlation coefficient for two clusterings.

__author__ = 'Alexander Junge (alexander.junge@gmail.com)'

from sklearn.metrics import matthews_corrcoef


def __clustering_to_classification__(clustering_result):
    """
    :param clustering_result: a list/tuple of strings, the string at position i is the cluster object i is assigned to
    :return: a list of integers containing +1 if current pair of objects belong to the same cluster and -1 otherwise
    """
    classification = []
    n = len(clustering_result)
    for obj1 in range(n):
        for obj2 in range(obj1 + 1, n):
            if clustering_result[obj1] == clustering_result[obj2]:
                classification.append(+1)
            else:
                classification.append(-1)
    return classification


def clustering_matthews_corrcoef(clusters_true, clusters_predicted):
    """
    Computes Matthew's correlation coefficient for a ground truth and predicted cluster assignment.
    Since Matthew's correlation coefficient is originally defined for a binary classification scenario,
    each clustering is first converted to a binary classification vector.
    This vector contains +1 for each pair of objects that are assigned to the same cluster and -1 for each pair
    that belongs to different clusters.

    :param clusters_true: a list/tuple of strings, the string at position i is the cluster object i is assigned to in
                          the ground truth clustering
    :param clusters_predicted: a list/tuple of strings, the string at position i is the cluster object i is assigned to
                          in the predicted clustering
    :return: Matthew's correlation coefficient for the true and predicted clustering
    """
    classification_true = __clustering_to_classification__(clusters_true)
    classification_predicted = __clustering_to_classification__(clusters_predicted)
    return matthews_corrcoef(classification_true, classification_predicted)