#!/usr/bin/env python

import os
import sys
import argparse

from Bio.Align import MultipleSeqAlignment
from Bio import AlignIO
'''
    merge_stk_to_fasta
    ~~~~~~~~~~~~~

Merge stockholm files and store them into a single fasta file
Useful to prepare input for GraphClust from the benchmark datasets  
@author:    Milad Miladi
            miladim@informatik.uni-freiburg.de

'''


import glob
import re
import os
import string
def stockholm2fasta(input_dir, pattern, outfile, min_num_subaligns):
    #files = sorted(glob.glob(input_dir + pattern))
    # sort files such that we get the order as bash ls command. RF00005.0, RF00005.10, RF00005.11, RF00005.1 
    alphabet = string.printable  # 0-9A-Za-z_-. 
    files_unsorted = glob.glob(input_dir + pattern)
    files = sorted( files_unsorted, key=lambda word: [alphabet.index(c) for c in word+"..."])

    
    # filter by min partition size
    count_dict = dict()
    for fl in files:
        family = re.match("\S*(RF\d+)\S*", fl).group(1)
        if family in count_dict:
            count_dict[family] += 1
        else:
            count_dict[family] = 1

    
    fasta_handle = open (outfile,'w')       
    for fl in files:
        family = re.match("\S*(RF\d+)\S*", fl).group(1)
        
        if count_dict[family] < min_num_subaligns:
            print "skipped: ", fl
            continue
        
        stk_handle = open (fl,'r')
        alignment = AlignIO.read(stk_handle, "stockholm")
        for record in alignment:
            record.id, record.seq
            fasta_handle.write(">{}_{} {}\n{}\n".format(os.path.basename(fl),record.id, os.path.basename(fl), 
                                                     record.seq.ungap("-")))
        stk_handle.close()

    fasta_handle.close()

def is_valid_path(arg):
    if not os.path.exists(arg):
        parser.error("Given path %s does not exist" % arg)
    return arg

# =========================================================================
# Main() section
# =========================================================================
print "Args: ", sys.argv

##invoke converter
##extract subalignment files, format eg.  RF00001.0, RF00001.29,...  
parser = argparse.ArgumentParser()
parser.add_argument('--dir', type=is_valid_path, required=True,
                    help='Directory for input stk files and out-fasta')
parser.add_argument('--out-fasta', type=str, default ="representatives",
                    help='Merged fasta name is stored')
parser.add_argument('--stk-pattern', type=str, default ="RF[0-9]*_rep.*",
                    help='Pattern to locate stockholm files, supports shell-style wildcards')
parser.add_argument('--min-num-subaligns', type=int, default = 3,
                    help='Discard families where number of subalignments is less than min')


args = parser.parse_args()

stockholm2fasta(args.dir,  args.stk_pattern, "{}/{}-minsize-{}-csort.fa".format(args.dir, args.out_fasta, args.min_num_subaligns), args.min_num_subaligns)


'''
#Next steps to run GarphClust, can be added to this script:
setup perl5    
#Below is a bash command make a gspan  outof AlignCLust bz2 files in a sorted order
    rm all.gspan.gz ; LC_COLLATE=C ls GSPAN/RF*.gspan.bz2 |  xargs -I {} bzcat {} | gzip > all.gspan.gz
    python bin/analysis/tools/filter_GSPAN.py all.gpan.gz --filtered-format > misize-3.gspan
    /usr/local/user/GraphClust-0.7.6/bin/NSPDK -R 3 -D 3 -gt DIRECTED -fg misize-3.gspan -of
    
#Sanity check for before running hybrid AC-BC shortcut (B):
diff <(cat results/20150720-alignclust-20150717-human-cliques/20150720-alignclust-20150717-human-cliques-misize-3.gspan | grep "t #" |  awk '{print $4}' | sed 's/_filtered//' | sed 's/.folded//') <(cat ../GraphClust/20150721-20150717-human-cliques/graphclust/maxSI-95/FASTA/data.names | awk '{print $6}'  | sed 's/_rep//')
  
'''


