#!/usr/bin/perl

use strict 'vars';
use warnings;
use Getopt::Long;
use Pod::Usage;

######################################################################
# This program gets as input an RNA secondary structure in dotbracket format 
# and embed the strcuture in the given dotplot file which has a post script format
# no compatibility check of length, etc is performed; dot plot and dotbracket must be compatible
# < and > are replace with ( and )
######################################################################

# command line options
my ($i_ps_file);
my $options = GetOptions ("ps=s" => \$i_ps_file);
($options) or pod2usage(2);
($i_ps_file) or pod2usage("Error: the option -ps <ps-file> is compulsory!\n");
( -e $i_ps_file ) or pod2usage("Error: no such file - $i_ps_file!\n");

my $struct_str ;#= ".............((...(((.((((........)))).)))...........((((......(((...(((..(((((......)))))......................)))...)))......))))....))...";
print STDOUT "Enter the secondary strcuture in dot bracket format\n";
$struct_str = <STDIN>;
$struct_str =~ tr/<>/()/;
my @struct_arr = split("",$struct_str);
#use array open as an stack for read (
my @open = ();
my @bps = ();
foreach my $idx (0..( @struct_arr-1 ) )
{
  if ( $struct_arr[$idx] eq "(" ) {
    push( @open, $idx );
  }
  elsif ( $struct_arr[$idx] eq ")" ) {
    if (@open) {
      ##either both ends should be remained or both should be filtered out 
      my $left_idx  = pop(@open)  ;
      push (@bps, ($left_idx) . " " . ($idx));
    }
    else  
    {	
      die "error in dotbracket" ;
    }
  }
}

print "bps: " . join ("; ", @bps)."\n";

modify_ps_file($i_ps_file);

sub modify_ps_file
{
  my $qfn = shift;
  if (@bps eq 0 ) 
  { 
    return;
  }
  ##modify the file
  rename($qfn, "$qfn.old")    or die "Can't rename \"$qfn\": $!\n";

  open(my $fh_in, '<', "$qfn.old")
	or die "Can't open \"$qfn\": $!\n";
  open(my $fh_out, '>', "$qfn")
	or die "Can't create \"$qfn\": $!\n";

  while (my $line = <$fh_in>) {
    chomp $line;
    if ($line =~ /\%embedded struct/)
    {
      print STDERR "WARNING: embedded struct overwritten\n";
      
      print $fh_out "\%embedded struct\n";
      print $fh_out "$_ 2.9500000 b\n" for @bps;
      print $fh_out  "showpage";
      close ($fh_in);
      close ($fh_out);
      return;
    }
    elsif ($line =~ /^showpage/)
    {
      print $fh_out "\%embedded struct\n";
      print $fh_out "$_ 2.9500000 b\n" for @bps;
      print $fh_out  "showpage";
      close ($fh_in);
      close ($fh_out);     
      return;
    }
    else   
    {
      print($fh_out "$line\n");
    }
  }
  unlink("$qfn.old");
}