#!/usr/bin/perl -w
# Copyright (C) 2014 Stefan S Seemann <seemann@rth.dk>
#
# filterRNAsecstruc.pl
# filter RNA secondary structure dotplots for different types of lonely basepairs

use strict;

sub usage {
    print <<EOT;
Usage: $0 dot-bracket-structure-FILE mode
Filter RNA secondary structure dotplots for different modes of lonely basepairs:

0 ... no single basepairs, e.g. (((.(...).))).
1 ... no stems of single basepairs, e.g. (((.(...).(...)))).
2 ... no stems of single basepairs in the backbone, e.g. (((.(...).(...))))..(...).

EOT
    exit;
}

# get options
&usage() if defined $ARGV[0] && $ARGV[0] eq "-h" || @ARGV != 2;

my $DEBUG = 0;

my ( $open, $first, $good, $close, $stack );
my $mode = $ARGV[1];

open IN, $ARGV[0] || die("Cannot open file!\n");

while( <IN> ) {
	$good = 1;
	$close = 0;
	$stack = 0;

	chomp $_;
	tr/-/./;
	my @l = split " ";
	my @p = split "", $l[0];
	my $bps = &getbasepairs(\@p);

	foreach my $l ( sort { $a <=> $b } keys %$bps ) {
		$open = ( $l < $$bps{$l} ) ? 1 : 0;

		if( $open ) {
			if( !$mode ) {
				if( !exists $$bps{$l-1} || $l-1 > $$bps{$l-1} ) {
					if( !exists $$bps{$l+1} && !exists $$bps{$$bps{$l}-1} ) {
						if( !exists $$bps{$$bps{$l}+1} || $$bps{$l}+1 < $$bps{$$bps{$l}+1} ) {	
							$good = 0;
							if( $DEBUG ) {
								print $l . " " . $$bps{$l} . " ";
								print "LONELY BASEPAIR\n";
							}
							last;
						}
					}
				}
			}

			$stack++;
			if( $close == 1 && ( $mode < 2 || ( $mode == 2 && !$stack ) ) ) {
				$good = 0;
				if( $DEBUG ) {
					print $l . " " . $$bps{$l} . " ";
					print "LONELY BASEPAIR because only one closed base pair\n";
				}
				last;
			}
			$first = ( $close ) ? 1 : 0;
			$close = 0;
		}
		else {
			$stack--;
			$close++;
			if( $first && ( $mode < 2 || ( $mode == 2 && !$stack ) ) ) {
				$good = 0;
				if( $DEBUG ) {
					print $l . " " . $$bps{$l} . " ";
					print "LONELY BASEPAIR because only one open base pair\n";
				}
				last;
			}
			$first = 0;
		}
	}

	print $_ . "\n" if $good;
}
close IN;

sub getbasepairs
{
    my ($struc) = @_;
    my %basepairs;
    my (@tmp, $base);

    for(my $i=0; $i<$#$struc+1; $i++) {
        if( $struc->[$i] eq '(' ) {
            push @tmp, $i;
        }
        elsif( $struc->[$i] eq ')' ) {
	    $base = pop @tmp;
            $basepairs{$base} = $i;
	    $basepairs{$i} = $base;
        }
    }

    die("Consensus structure invalid: number of left and right parenthesis unequal!\n") if $#tmp>-1;

    return \%basepairs;
}

