package MultipleAlignment::PETfold;
require Exporter;
use strict 'vars';
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw(read_stockholm
			);
#@EXPORT_OK = qw(			);


##############

#######################################################################
# Returns 
# INPUT: 
# OUTPUT: 
#######################################################################


my $STRUCT_PREFIX = "PETfold RNA structure:";


###############################################################################
# METHODS
###############################################################################
#write the struct to file
sub write_struct_to_file
{
	my ($file_name, $str) = @_;
	open (OUT_HANDLE, ">", $file_name);
	print OUT_HANDLE "$str\n";
}

## filter structure based on reliabilities
sub filter_structure
{
	my ($struct, $reli, $threshold) = @_;
	my @struct_arr = split (//, $struct);
	my @rel_arr = split (/\s+/, $reli);
	(@struct_arr == @rel_arr) or die "ERROR in filter_structure: structure and reliablities do not match\n";
	my @filtered_struct_arr  ;
	foreach my $idx (0..( @rel_arr-1 ) )
	{
#		print "$idx: $rel_arr[$idx]  $struct_arr[$idx]\n";
		
	#todo: WARNING filtering should be based on a basepair on relibility of individual base positions
		if ( $rel_arr[$idx] < $threshold  && ($struct_arr[$idx] =~ /[\(\)]/))
		{
			# base has is lower than threshold and should be filterd out
			$filtered_struct_arr[$idx] = ".";
		}
		else
		{
			$filtered_struct_arr[$idx] = $struct_arr[$idx];
#			if ($struct_arr[$idx] =~ /[\(\)]/) {
#				print $rel_arr[$idx] . " < " .  $threshold  . "--" ;
#			}
		}
#		print "\n";
	}
	
	print "Reliable filter is:\n" . join ('',@filtered_struct_arr) ."\n";
	
	#filter possibly halfly removed basepair, this should be resolved if above todo be done
	#check cons and filtered structure using stacks, so no crossing base pair is allowed
	my ( @open, @close ) = ();
	
	foreach my $idx (0..( @struct_arr-1 ) )
	{
		if ( $struct_arr[$idx] eq "(" ) {
			push( @open, $idx );
		}
		elsif ( $struct_arr[$idx] eq ")" ) {
			if (@open) {
				##either both ends should be remained or both should be filtered out 
				 my $left_idx  = pop(@open)  ;
				 if ( $filtered_struct_arr[$left_idx] ne "(" 
				 		or  $filtered_struct_arr[$idx] ne ")")
		 		{
		 			## make sure both basepair ends are removed 
		 			$filtered_struct_arr[$left_idx] = ".";
		 			$filtered_struct_arr[$idx] = ".";
		 		}
			}
			else {
				die("ERROR: The given cons structure has the wrong format, i.e. \n"
					  . "there are an uneven number of opening and closing brackets: $struct!\n"
				);
			}
		}
	}
	return join ('',@filtered_struct_arr);
}

##parse  output log (i.e. redirected stdout ) of PETfold
sub parse_outlog
{
	my ($log_file) = @_;
	open (my $in_handle, "<", $log_file) or die "Failed to open: $!";

	my $cons_struct = "";
	# Reading from that filehandle
	while(my $line = <$in_handle>) {
		if ($line =~ /^$STRUCT_PREFIX\s+(.+)/)
		{
			$cons_struct = $1;
			chomp($cons_struct);
			print "cons struct found:\n$cons_struct\n";
			return $cons_struct;
		}
	}	
	
	die "ERROR parse_PETfold_outlog: no cons struct found in $log_file\n";
}


##parse reliability pp file generated by PETfold
sub parse_reali_pp
{
	my ($pp_file) = @_;
	open (my $in_handle, "<", $pp_file) or die "Failed to open: $!";
	my $line_count = 0;
	my $alignment_len;
	my $cons_rels = "";
	# Reading from that filehandle
	while(my $line = <$in_handle>) {
	        $line_count++;
	        if ($line_count == 1)
	        {
	        	$alignment_len = $line;
#	        	print "alignment_len is $alignment_len\n" if ($i_debug);
	        }
	        else 
	        {
	        	# last line is consensus reliablity?
	        	if ($line_count == $alignment_len+3) #todo additional check for new lines
	        	{
	        		$cons_rels = $line;
#	        		print "consensus reliabilities are:\n $cons_rels\n" if ($i_debug);
	        	}
	        }
	}
	($cons_rels ne "" ) or die "consensus reliabilities could not be read from $pp_file\n";
	return $cons_rels;	
}


######################################################################
# Parse PETfold outfold and return struture (i.e. first line) 
# INPUT: Logfile
# OUTPUT: 
#######################################################################
sub parse_petfold_cons_file
{
	my ($file_name) = @_;
	open (IN_HANDLE, "<", $file_name);
	my $line = <IN_HANDLE>;
	chomp ($line);
	return $line;	
}


